@@ -6,7 +6,7 @@ from django.test import TestCase
 
 from bulk_sync import bulk_compare
 from bulk_sync import bulk_sync
-from .models import Company, Employee
+from .models import Company, Employee, EmployeeDifferentPk
 
 
 class BulkSyncTests(TestCase):
@@ -70,6 +70,61 @@ class BulkSyncTests(TestCase):
         self.assertEqual(50, new_e5.age)
         self.assertEqual(c1, new_e5.company)
 
+    def test_all_features_at_once_nonstandard_pk(self):
+        c1 = Company.objects.create(name="Foo Products, Ltd.")
+        c2 = Company.objects.create(name="Bar Microcontrollers, Inc.")
+
+        e1 = EmployeeDifferentPk.objects.create(name="Scott", age=40, company=c1)
+        e2 = EmployeeDifferentPk.objects.create(name="Isaac", age=9, company=c1)
+        e3 = EmployeeDifferentPk.objects.create(name="Zoe", age=9, company=c1)
+        e4 = EmployeeDifferentPk.objects.create(name="Bob", age=25, company=c2)
+
+        # We should update Scott's and Isaac's age, delete Zoe, add Newguy and
+        # add a second Bob (since he's not in company c1, which we filtered on.)
+        new_objs = [
+            EmployeeDifferentPk(name="Scott", age=41, company=c1),
+            EmployeeDifferentPk(name="Isaac", age=9, company=c1),
+            EmployeeDifferentPk(name="Newguy", age=10, company=c1),
+            EmployeeDifferentPk(name="Bob", age=50, company=c1),
+        ]
+
+        ret = bulk_sync(new_models=new_objs, filters=Q(company_id=c1.id), key_fields=("name",))
+
+        self.assertEqual(2, ret["stats"]["updated"])
+        self.assertEqual(2, ret["stats"]["created"])
+        self.assertEqual(1, ret["stats"]["deleted"])
+
+        self.assertEqual(4, EmployeeDifferentPk.objects.filter(company=c1).count())
+        self.assertEqual(1, EmployeeDifferentPk.objects.filter(company=c2).count())
+
+        new_e1 = EmployeeDifferentPk.objects.get(employee_id=e1.employee_id)
+        self.assertEqual("Scott", new_e1.name)
+        self.assertEqual(41, new_e1.age)
+        self.assertEqual(c1, new_e1.company)
+
+        new_e2 = EmployeeDifferentPk.objects.get(employee_id=e2.employee_id)
+        self.assertEqual("Isaac", new_e2.name)
+        self.assertEqual(9, new_e2.age)
+        self.assertEqual(c1, new_e2.company)
+
+        with self.assertRaises(EmployeeDifferentPk.DoesNotExist):
+            EmployeeDifferentPk.objects.get(employee_id=e3.employee_id)
+
+        new_e4 = EmployeeDifferentPk.objects.get(employee_id=e4.employee_id)
+        self.assertEqual("Bob", new_e4.name)
+        self.assertEqual(25, new_e4.age)
+        self.assertEqual(c2, new_e4.company)
+
+        new_e3 = EmployeeDifferentPk.objects.get(name="Newguy")
+        self.assertEqual("Newguy", new_e3.name)
+        self.assertEqual(10, new_e3.age)
+        self.assertEqual(c1, new_e3.company)
+
+        new_e5 = EmployeeDifferentPk.objects.get(name="Bob", company=c1)
+        self.assertEqual("Bob", new_e5.name)
+        self.assertEqual(50, new_e5.age)
+        self.assertEqual(c1, new_e5.company)
+
     def test_provided_pk_is_retained_but_raises_if_mismatch_with_keyfield(self):
         c1 = Company.objects.create(name="Foo Products, Ltd.")
         e1 = Employee.objects.create(name="Scott", age=40, company=c1)
@@ -219,30 +274,73 @@ class BulkSyncTests(TestCase):
         ret = bulk_sync(new_models=[], filters=None, key_fields=("name",), db_class=Employee)
         ret = bulk_sync(new_models=Employee.objects.none(), filters=None, key_fields=("name",))
 
+    def test_new_objs_with_unprepped_field_values_are_processed_correctly(self):
+        c1 = Company.objects.create(name="Foo Products, Ltd.")
+        c2 = Company.objects.create(name="Bar Microcontrollers, Inc.")
+        e1 = Employee.objects.create(name="Scott", age=40, company=c1)
+
+        new_objs = [Employee(name="Scott", age="40", company=c2)]
+
+        ret = bulk_sync(
+            new_models=new_objs,
+            filters=None,
+            key_fields=("name", "age"),
+        )
+
+        # we should should update e1's company to c2
+        self.assertEqual(1, ret["stats"]["updated"])
+        self.assertEqual(c2, Employee.objects.get(pk=e1.pk).company)
+
+        # we should not create or delete anything
+        self.assertEqual(0, ret["stats"]["created"])
+        self.assertEqual(0, ret["stats"]["deleted"])
+        self.assertEqual(1, Employee.objects.count())
+
 
 class BulkCompareTests(TestCase):
     """ Test `bulk_compare` method """
 
-    @classmethod
-    def setUpTestData(cls):
-        cls.c1 = Company.objects.create(name="Foo Products, Ltd.")
-        cls.c2 = Company.objects.create(name="Bar Microcontrollers, Inc.")
+    # @classmethod
+    # def setUpTestData(cls):
+    def setupEmployees(self):
+        self.c1 = Company.objects.create(name="Foo Products, Ltd.")
+        self.c2 = Company.objects.create(name="Bar Microcontrollers, Inc.")
 
-        cls.e1 = Employee.objects.create(name="Scott", age=40, company=cls.c1)
-        cls.e2 = Employee.objects.create(name="Isaac", age=9, company=cls.c1)
-        cls.e3 = Employee.objects.create(name="Zoe", age=9, company=cls.c1)
-        cls.e4 = Employee.objects.create(name="Bob", age=25, company=cls.c2)
+        self.e1 = Employee.objects.create(name="Scott", age=40, company=self.c1)
+        self.e2 = Employee.objects.create(name="Isaac", age=9, company=self.c1)
+        self.e3 = Employee.objects.create(name="Zoe", age=9, company=self.c1)
+        self.e4 = Employee.objects.create(name="Bob", age=25, company=self.c2)
 
         # We should update Scott's and Isaac's age, delete Zoe, add Newguy and
         # add a second Bob (since he's not in company c1, which we filtered on.)
-        cls.new_objs = [
-            Employee(name="Scott", age=41, company=cls.c1),
-            Employee(name="Isaac", age=9, company=cls.c1),
-            Employee(name="Newguy", age=10, company=cls.c1),
-            Employee(name="Bob", age=50, company=cls.c1),
+        self.new_objs = [
+            Employee(name="Scott", age=41, company=self.c1),
+            Employee(name="Isaac", age=9, company=self.c1),
+            Employee(name="Newguy", age=10, company=self.c1),
+            Employee(name="Bob", age=50, company=self.c1),
+        ]
+
+    def setupEmployeeDiffPk(self):
+        self.c1 = Company.objects.create(name="Foo Products, Ltd.")
+        self.c2 = Company.objects.create(name="Bar Microcontrollers, Inc.")
+
+        self.e1 = EmployeeDifferentPk.objects.create(name="Scott", age=40, company=self.c1)
+        self.e2 = EmployeeDifferentPk.objects.create(name="Isaac", age=9, company=self.c1)
+        self.e3 = EmployeeDifferentPk.objects.create(name="Zoe", age=9, company=self.c1)
+        self.e4 = EmployeeDifferentPk.objects.create(name="Bob", age=25, company=self.c2)
+
+        # We should update Scott's and Isaac's age, delete Zoe, add Newguy and
+        # add a second Bob (since he's not in company c1, which we filtered on.)
+        self.new_objs = [
+            EmployeeDifferentPk(name="Scott", age=41, company=self.c1),
+            EmployeeDifferentPk(name="Isaac", age=9, company=self.c1),
+            EmployeeDifferentPk(name="Newguy", age=10, company=self.c1),
+            EmployeeDifferentPk(name="Bob", age=50, company=self.c1),
         ]
 
     def test_bulk_compare(self):
+        self.setupEmployees()
+
         c1 = self.c1
         e3 = self.e3
         new_objs = self.new_objs
@@ -256,6 +354,8 @@ class BulkCompareTests(TestCase):
         self.assertEqual([new_objs[1]], ret["unchanged"])
 
     def test_bulk_compare_with_ignore_int_field(self):
+        self.setupEmployees()
+
         c1 = self.c1
         e3 = self.e3
         new_objs = self.new_objs
@@ -274,6 +374,8 @@ class BulkCompareTests(TestCase):
         self.assertEqual([new_objs[0], new_objs[1]], ret["unchanged"])
 
     def test_bulk_compare_with_ignore_relation_field(self):
+        self.setupEmployees()
+
         c1 = self.c1
         e3 = self.e3
         new_objs = self.new_objs
@@ -290,3 +392,23 @@ class BulkCompareTests(TestCase):
         self.assertEqual([new_objs[0]], ret["updated"])
         self.assertEqual({new_objs[0]: {"age": (40, 41)}}, ret["updated_details"])
         self.assertEqual([new_objs[1]], ret["unchanged"])
+
+    def test_nonstandard_pk(self):
+        self.setupEmployeeDiffPk()
+
+        c1 = self.c1
+        e3 = self.e3
+        new_objs = self.new_objs
+
+        ret = bulk_compare(
+            old_models=EmployeeDifferentPk.objects.filter(company=c1).order_by("name"),
+            new_models=new_objs,
+            key_fields=("name",),
+            ignore_fields=("company_id",),
+        )
+
+        self.assertEqual([new_objs[2], new_objs[3]], ret["added"])
+        self.assertEqual([e3], list(ret["removed"]))
+        self.assertEqual([new_objs[0]], ret["updated"])
+        self.assertEqual({new_objs[0]: {"age": (40, 41)}}, ret["updated_details"])
+        self.assertEqual([new_objs[1]], ret["unchanged"])
